import { world, system } from "@minecraft/server";
import * as blockUtilities from "../utilities/block_utilities.js"

world.beforeEvents.worldInitialize.subscribe(({blockComponentRegistry}) => {
	blockComponentRegistry.registerCustomComponent("megaverse:block_chair_sit_default", BlockChairSitDefault);
	blockComponentRegistry.registerCustomComponent("megaverse:block_crop", BlockCrop);
	blockComponentRegistry.registerCustomComponent("megaverse:block_egg", BlockEgg);
	blockComponentRegistry.registerCustomComponent("megaverse:block_flower", BlockFlower);
	blockComponentRegistry.registerCustomComponent("megaverse:block_sapling", BlockSapling);
	blockComponentRegistry.registerCustomComponent("megaverse:block_small_shutter", BlockSmallShutter);
	blockComponentRegistry.registerCustomComponent("megaverse:block_trap_cover", BlockTrapCover);
	blockComponentRegistry.registerCustomComponent("megaverse:break_firewood", BreakFirewood);
	blockComponentRegistry.registerCustomComponent("megaverse:cryothermalia", Cryothermalia);
	blockComponentRegistry.registerCustomComponent("megaverse:deer_skull", DeerSkull);
	blockComponentRegistry.registerCustomComponent("megaverse:frost", Frost);
	blockComponentRegistry.registerCustomComponent("megaverse:frosted_block", FrostedBlock);
	blockComponentRegistry.registerCustomComponent("megaverse:ice_spikes", IceSpikes);
	blockComponentRegistry.registerCustomComponent("megaverse:interact_block_egg", InteractBlockEgg);
	blockComponentRegistry.registerCustomComponent("megaverse:interact_firewood", InteractFirewood);
	blockComponentRegistry.registerCustomComponent("megaverse:kindling_pile", KindlingPile);
	blockComponentRegistry.registerCustomComponent("megaverse:random_tick_ice_spikes", RandomTickIceSpikes);
	blockComponentRegistry.registerCustomComponent("megaverse:rock_fern", RockFern);
	blockComponentRegistry.registerCustomComponent("megaverse:sconce", Sconce);
	blockComponentRegistry.registerCustomComponent("megaverse:silkworm_cocoon", SilkwormCocoon);
	blockComponentRegistry.registerCustomComponent("megaverse:tick_block_egg", TickBlockEgg);
	blockComponentRegistry.registerCustomComponent("megaverse:tick_frosted_block", TickFrostedBlock);
});

const BlockCrop = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.blockCropInteract(block,dimension,player);
	},
	onRandomTick({block,dimension}) {
		blockUtilities.blockCropRandomTick(block,dimension);
	}
};
const BlockEgg = {
	onEntityFallOn({block,dimension}) {
		blockUtilities.blockEggFallOn(block,dimension);
	},
	onPlayerDestroy({block,destroyedBlockPermutation,dimension,player}) {
		blockUtilities.blockEggBreak(block,destroyedBlockPermutation,dimension,player);
	}
};
const BlockFlower = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.blockFlowerInteract(block,dimension,player);
	}
};
const BlockSapling = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.blockSaplingInteract(block,dimension,player);
	},
	onRandomTick({block,dimension}) {
		blockUtilities.blockSaplingRandomTick(block,dimension);
	}
};
const BlockSmallShutter = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.blockSmallShutterInteract(block,dimension,player);
	}
};

const BlockTrapCover = {
	onEntityFallOn({block,dimension}) {
		blockUtilities.blockTrapCoverFallOn(block,dimension);
	},
	onStepOn({block,dimension,entity}) {
		blockUtilities.blockTrapCoverStepOn(block,dimension,entity);
	}
};


const BlockChairSitDefault = {
	onPlayerInteract({block,dimension,player}) {
		const tileEntityId = "megaverse:chair_block";
		const sitHeight = 0.4;
		blockUtilities.blockChairSatDefault(block,dimension,player,tileEntityId,sitHeight);
	}
};

const BlockPreciseRotation = {
	onPlayerInteract({block,dimension,player}) {
		const tileEntityId = "megaverse:chair_block";
		const sitHeight = 0.4;
		blockUtilities.blockChairSatDefault(block,dimension,player,tileEntityId,sitHeight);
	}
};

const BreakFirewood = {
	onPlayerDestroy({block,destroyedBlockPermutation,dimension,player}) {
		blockUtilities.firewoodBreak(block,destroyedBlockPermutation,dimension,player);
	}
};

const Cryothermalia = {
	onRandomTick({block,dimension}) {
		blockUtilities.cryothermaliaRandomTick(block,dimension);
	},
	onTick({block,dimension}) {
		blockUtilities.cryothermaliaTick(block,dimension);
	}
};

const DeerSkull = {
	onPlayerInteract({block,dimension,face,player}) {
		blockUtilities.deerSkullInteract(block,dimension,face,player);
	},
	onPlayerDestroy({block,destroyedBlockPermutation,dimension,player}) {
		blockUtilities.deerSkullBreak(block,destroyedBlockPermutation,dimension,player);
	}
};

const Frost = {
	onRandomTick({block,dimension}) {
		blockUtilities.frostRandomTick(block,dimension);
	},
	onTick({block,dimension}) {
		blockUtilities.frostTick(block,dimension);
	}
};

const FrostedBlock = {
	onRandomTick({block,dimension}) {
		blockUtilities.frostedBlockRandomTick(block,dimension);
	}
};

const IceSpikes = {
	onEntityFallOn({block,dimension}) {
		blockUtilities.iceSpikesFallOn(block,dimension);
	},
	onStepOn({block,dimension,entity}) {
		blockUtilities.iceSpikesStepOn(block,dimension,entity);
	}
};

const InteractBlockEgg = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.blockEggInteract(block,dimension,player);
	}
};
const InteractFirewood = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.firewoodInteract(block,dimension,player);
	}
};
const KindlingPile = {
	onPlayerDestroy({block,destroyedBlockPermutation,dimension,player}) {
		blockUtilities.kindlingPileBreak(block,destroyedBlockPermutation,dimension,player);
	},
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.kindlingPileInteract(block,dimension,player);
	}
};
const RandomTickIceSpikes = {
	onRandomTick({block,dimension}) {
		blockUtilities.iceSpikesRandomTick(block,dimension);
	}
};
const RockFern = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.rockFernInteract(block,dimension,player);
	}
};
const Sconce = {
	onPlayerInteract({block,dimension,player}) {
		blockUtilities.sconceInteract(block,dimension,player);
	},
	onRandomTick({block,dimension}) {
		blockUtilities.sconceRandomTick(block,dimension);
	}
};
const SilkwormCocoon = {
	onTick({block,dimension}) {
		blockUtilities.silkwormCocoonTick(block,dimension);
	}
};
const TickBlockEgg = {
	onTick({block,dimension}) {
		blockUtilities.blockEggTick(block,dimension);
	}
};
const TickFrostedBlock = {
	onTick({block,dimension}) {
		blockUtilities.frostedBlockTick(block,dimension);
	}
};