import { world, system, EquipmentSlot, ItemStack, Player } from "@minecraft/server";
import * as UIComponents from "./ui_components.js"
import * as itemArrays from "../arrays/item_arrays.js"
import * as itemUtilities from "../utilities/item_utilities.js"

world.beforeEvents.worldInitialize.subscribe(({itemComponentRegistry}) => {
	itemComponentRegistry.registerCustomComponent("megaverse:consume_dracon_acid_bottle", ConsumeDraconAcidBottle);
	itemComponentRegistry.registerCustomComponent("megaverse:consume_raw_meat", ConsumeRawMeat);
	itemComponentRegistry.registerCustomComponent("megaverse:consume_ice_wyvern_venomous_frost_bottle", ConsumeIceWyvernVenomousFrostBottle);
	itemComponentRegistry.registerCustomComponent("megaverse:hadrosaurid_horn", HadrosauridHorn);
	itemComponentRegistry.registerCustomComponent("megaverse:item_tool_durability", ItemToolDurability);
	itemComponentRegistry.registerCustomComponent("megaverse:item_weapon_durability", ItemWeaponDurability);
	itemComponentRegistry.registerCustomComponent("megaverse:otherworld_mirror", OtherworldMirror);
	itemComponentRegistry.registerCustomComponent("megaverse:otherworld_sword", OtherworldSword);
	itemComponentRegistry.registerCustomComponent("megaverse:use_alchemy_wand", UseAlchemyWand);
});

const ConsumeDraconAcidBottle = {
	onConsume({itemStack,source}) {
		itemUtilities.draconAcidBottleConsume(itemStack,source);
	}
};
const ConsumeRawMeat = {
	onConsume({itemStack,source}) {
		itemUtilities.rawMeatConsume(itemStack,source);
	}
};
const ConsumeIceWyvernVenomousFrostBottle = {
	onConsume({itemStack,source}) {
		itemUtilities.iceWyvernVenomousFrostBottleConsume(itemStack,source);
	}
};
const HadrosauridHorn = {
	onUse({itemStack,source}) {
		itemUtilities.hadrosauridHorn(itemStack,source);
	}
};

const ItemToolDurability = {
	onBeforeDurabilityDamage({attackingEntity:player,itemStack}) {
		itemUtilities.itemToolDamage(itemStack,player);
	},
	onMineBlock({itemStack,source:player}) {
		itemUtilities.itemToolDamage(itemStack,player);
	}
};

const ItemWeaponDurability = {
	onMineBlock({itemStack,source:player}) {
		itemUtilities.itemToolDamage(itemStack,player);
	}
};

const OtherworldMirror = {
	onUse({source}) {
		const mainhand = source.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.isSneaking==false) {
			source.playSound('vault.activate',{location:source.location});
			UIComponents.OtherworldMirrorUI(source);
		}
		if (source.isSneaking) {
			source.dimension.spawnParticle("megaverse:otherworld_mirror_summoning", source.location);
			source.playSound('vault.activate',{location:source.location});
			mainhand.setItem(new ItemStack("megaverse:otherworld_sword"));
		}
	},
	onHitEntity({attackingEntity,hadEffect,hitEntity}) {
		const mainhand = attackingEntity.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (attackingEntity.isSprinting) {
			hitEntity.addEffect("slowness",100,{amplifier:2,showParticles:false});
			hitEntity.setOnFire(60,true);
		}
		hitEntity.applyImpulse({
			x:attackingEntity.getViewDirection().x+attackingEntity.getVelocity().x,
			y:attackingEntity.getViewDirection().y+(-attackingEntity.getVelocity().y/2),
			z:attackingEntity.getViewDirection().z+attackingEntity.getVelocity().z
		});
		attackingEntity.startItemCooldown("otherworld_sword",20);
		attackingEntity.playSound('vault.activate',{location:attackingEntity.location});
		mainhand.setItem(new ItemStack("megaverse:otherworld_sword"));
	},
	onUseOn({source}) {
		const mainhand = source.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.isSneaking) {
			if (source.getItemCooldown("otherworld_mirror")==0) {
				source.dimension.spawnParticle("megaverse:otherworld_mirror_restoration", source.location);
				source.playSound('vault.activate',{location:source.location});
				mainhand.setItem(new ItemStack("megaverse:otherworld_sword"));
			}
		}
	}
};

const OtherworldSword = {
	onHitEntity({attackingEntity,hadEffect,hitEntity}) {
		const mainhand = attackingEntity.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (attackingEntity.isSprinting) {
			hitEntity.addEffect("slowness",100,{amplifier:2,showParticles:false});
			hitEntity.setOnFire(60,true);
		}
		hitEntity.applyImpulse({
			x:attackingEntity.getViewDirection().x+attackingEntity.getVelocity().x,
			y:attackingEntity.getViewDirection().y+(-attackingEntity.getVelocity().y/2),
			z:attackingEntity.getViewDirection().z+attackingEntity.getVelocity().z
		});
		attackingEntity.startItemCooldown("otherworld_sword",20);
	},
	onUse({source}) {
		const mainhand = source.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.isSneaking) {
			source.dimension.spawnParticle("megaverse:otherworld_mirror_restoration", source.location);
			source.playSound('vault.deactivate',{location:source.location});
			mainhand.setItem(new ItemStack("megaverse:otherworld_mirror"));
		}
	}
};
const UseAlchemyWand = {
	onCompleteUse({itemStack,source}) { 
		const entry = itemArrays.alchemyWandClassMagicTypes.find(type=>type.matches(itemStack.typeId));
		const config = {...itemArrays.alchemyWandDefaultMagicType,...(entry??{})};
		itemUtilities.alchemyWandCompleteUse(itemStack,source);
		itemUtilities.magicSphereProjectile(itemStack,source,config.soundType,config.entityType,config.entityEvent,config.magicType,config.castTimer,config.velocity);
	}
};