import { system, world, GameMode, Player } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData } from "@minecraft/server-ui";
import * as UIUtilities from "../utilities/ui_utilities.js"

export function OtherworldMirrorUI(source) {
	const main_menu = new ActionFormData();
	main_menu.title({ translate: "ui.megaverse:otherworld_mirror.title"});
	main_menu.body(
		{rawtext:[
			{translate:"ui.megaverse:otherworld_mirror.main_menu.text1"},
			{text:" "},
			{translate:`${source.name}`},
			{text:". "},{text:"\n"},
			{text:"\n"},
			{translate:"ui.megaverse:otherworld_mirror.main_menu.text3"}
		]}
	);
	main_menu.button({ translate: "ui.megaverse:otherworld_mirror.information.title" },"textures/ui/megaverse/otherworld_mirror/information");
	main_menu.button({ translate: "ui.megaverse:otherworld_mirror.summon.title" },"textures/ui/megaverse/otherworld_mirror/summon");
	main_menu.button({ translate: "ui.megaverse:otherworld_mirror.restoration.title" },"textures/ui/megaverse/otherworld_mirror/restoration");
	main_menu.button({ translate: "ui.megaverse:otherworld_mirror.summon.operator.title" },"textures/ui/megaverse/otherworld_mirror/summon_operator");
	main_menu.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		if (response.selection==0) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorMainInformationUI(source);
		}
		if (response.selection==1) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorMainSummoningUI(source);
		}
		if (response.selection==2) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorMainRestorationUI(source);
		}
		if (response.selection==3) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorOperatorSummoningUI(source);
		}
	});
};

function OtherworldMirrorMainInformationUI(source) {
	const information = new ActionFormData();
	information.title({ translate: "ui.megaverse:otherworld_mirror.information.title"});
	information.body(
		{rawtext:[
			{translate:"ui.megaverse:otherworld_mirror.main_menu.text2"},
			{text:"\n"},
			{text:"© 2025 Kyuu Desperation. All Rights Reserved."}
		]}
	);
	information.button({ translate: "ui.megaverse:otherworld_mirror.return" },"textures/ui/megaverse/otherworld_mirror/return");
	information.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		if (response.selection==0) {
			source.playSound('vault.close_shutter',{location:source.location});
			OtherworldMirrorUI(source);
		}
	});
};

function OtherworldMirrorMainRestorationUI(source) {
	const OtherworldMirrorModalUI = new ModalFormData();
	const healthComponent = source.getComponent("health");
	const currentHealthValue = healthComponent.currentValue;
	const defaultHealthValue = healthComponent.defaultValue;
	OtherworldMirrorModalUI.title({ translate: "ui.megaverse:otherworld_mirror.restoration.title"});
	OtherworldMirrorModalUI.slider({translate: "ui.megaverse:otherworld_mirror.restoration.health"}, currentHealthValue, defaultHealthValue, 1);
	OtherworldMirrorModalUI.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		else {
			const healingAmount = response.formValues;
			if (healingAmount==0) {
				source.sendMessage({translate: "ui.megaverse:otherworld_mirror.restoration.undefined"});
				return source.playSound('vault.deactivate',{location:source.location});
			}
			if (healingAmount==currentHealthValue&&currentHealthValue!=defaultHealthValue) {
				source.sendMessage({translate: "ui.megaverse:otherworld_mirror.restoration.same"});
				return source.playSound('magic.essentia_magic.default_impact',{location:source.location});
			}
			if (currentHealthValue==defaultHealthValue) {
				source.sendMessage({translate: "ui.megaverse:otherworld_mirror.restoration.max"});
				return source.playSound('magic.essentia_magic.default_impact',{location:source.location});
			}
			const levelConvert = (healingAmount*1)-(currentHealthValue*1);
			if (source.getGameMode()!=GameMode.creative) {
				if (source.level<levelConvert) {
					source.sendMessage({rawtext:[{translate:`${source.name}`},{text:", "},{translate:`${levelConvert}`},{text:" "},{translate:"ui.megaverse:otherworld_mirror.summon.requirements"}]});
					return source.playSound('vault.deactivate',{location:source.location});
				}
				source.addLevels(-(healingAmount*1)+(currentHealthValue*1));
			}
			source.playSound('magic.essentia_magic.default_cast',{location:source.location});
			source.dimension.spawnParticle("megaverse:otherworld_mirror_restoration", source.location);
			healthComponent.setCurrentValue(1*healingAmount);
			source.camera.fade({
				fadeColor: { red: 1, green: 1, blue: 1 },
				fadeTime: { fadeInTime: 0.1, holdTime: 0.0, fadeOutTime: 0.1 }
			});
			source.startItemCooldown("otherworld_mirror",20*5);
		}
	});
};

function OtherworldMirrorOperatorSummoningUI(source) {
	const OtherworldMirrorModalUI = new ModalFormData();
	OtherworldMirrorModalUI.title({ translate: "ui.megaverse:otherworld_mirror.title"});
	OtherworldMirrorModalUI.textField({translate: "ui.megaverse:otherworld_mirror.summon.title"}, {translate: "ui.megaverse:otherworld_mirror.summon.text"},"");
	OtherworldMirrorModalUI.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		else {
			const summoning = response.formValues;
			let selectedSummonId; let selectedSummonName; let requiredXP; let firstEvent; let secondEvent;
			if (summoning=="") {
				source.sendMessage({translate: "ui.megaverse:otherworld_mirror.summon.undefined"});
				return source.playSound('vault.deactivate',{location:source.location});
			}
			else {
				selectedSummonId = `${summoning}`;
				selectedSummonName = summoning;
				requiredXP = `${summoning.length}`;
				firstEvent = false;
				secondEvent = false;
				source.playSound('vault.open_shutter',{location:source.location});
				OtherworldMirrorChosenSummoningUI(source,selectedSummonId,selectedSummonName,requiredXP,firstEvent,secondEvent);
			}
		}
	});
};

function OtherworldMirrorMainSummoningUI(source) {
	const OtherworldMirrorActionUI = new ActionFormData();
	OtherworldMirrorActionUI.title({ translate: "ui.megaverse:otherworld_mirror.summon.title"});
	OtherworldMirrorActionUI.body({ translate: "ui.megaverse:otherworld_mirror.summon.text"});
	OtherworldMirrorActionUI.button({ translate: "ui.megaverse:otherworld_mirror.summon.title.creatures" },"textures/ui/megaverse/otherworld_mirror/creatures");
	OtherworldMirrorActionUI.button({ translate: "ui.megaverse:otherworld_mirror.summon.title.vehicles" },"textures/ui/megaverse/otherworld_mirror/vehicles");
	OtherworldMirrorActionUI.button({ translate: "ui.megaverse:otherworld_mirror.return" },"textures/ui/megaverse/otherworld_mirror/return");
	OtherworldMirrorActionUI.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		if (response.selection==0) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorCreatureSummoningUI(source);
		}
		if (response.selection==1) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorVehicleSummoningUI(source);
		}
		if (response.selection>=2) {
			source.playSound('vault.close_shutter',{location:source.location});
			OtherworldMirrorUI(source);
		}
	});
};

function OtherworldMirrorCreatureSummoningUI(source) {
	const OtherworldMirrorActionUI = new ActionFormData();
	OtherworldMirrorActionUI.title({ translate: "ui.megaverse:otherworld_mirror.summon.title.creatures"});
	OtherworldMirrorActionUI.body({ translate: "ui.megaverse:otherworld_mirror.summon.text.choose"});
	OtherworldMirrorActionUI.button({ translate: "entity.megaverse:silkmoth.name" },"textures/megaverse_items/default_nature/spawn_egg_nature/silkmoth_spawn_egg");
	OtherworldMirrorActionUI.button({ translate: "entity.megaverse:silkworm.name" },"textures/megaverse_items/default_nature/spawn_egg_nature/silkworm_spawn_egg");
	OtherworldMirrorActionUI.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		let selectedSummonId; let selectedSummonName; let requiredXP; let firstEvent; let secondEvent;
		if (response.selection==0) {
			selectedSummonId = "megaverse:silkmoth";
			selectedSummonName = "entity.megaverse:silkmoth.name";
			requiredXP = 10;
			firstEvent = false;
			secondEvent = false;
		}
		if (response.selection==1) {
			selectedSummonId = "megaverse:silkworm";
			selectedSummonName = "entity.megaverse:silkworm.name";
			requiredXP = 5;
			firstEvent = false;
			secondEvent = false;
		}
		if (response.selection>=2) {
			selectedSummonId = undefined;
			selectedSummonName = undefined;
			requiredXP = 0;
			firstEvent = false;
			secondEvent = false;
		}
		if (selectedSummonId!=undefined) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorChosenSummoningUI(source,selectedSummonId,selectedSummonName,requiredXP,firstEvent,secondEvent);
		}
		else {
			source.playSound('vault.close_shutter',{location:source.location});
			OtherworldMirrorMainSummoningUI(source);
		}
	});
};

function OtherworldMirrorVehicleSummoningUI(source) {
	const OtherworldMirrorActionUI = new ActionFormData();
	OtherworldMirrorActionUI.title({ translate: "ui.megaverse:otherworld_mirror.summon.title.vehicles"});
	OtherworldMirrorActionUI.body({ translate: "ui.megaverse:otherworld_mirror.summon.text.choose"});
	OtherworldMirrorActionUI.button({ translate: "item.megaverse:motorbike.name" },"textures/megaverse_items/default_equipment/advanced_equipment/motorbike");
	OtherworldMirrorActionUI.button({ translate: "item.megaverse:motorbike_extra_seat.name" },"textures/megaverse_items/default_equipment/advanced_equipment/motorbike_extra_seat");
	OtherworldMirrorActionUI.button({ translate: "item.megaverse:rotorcraft.name" },"textures/megaverse_items/default_equipment/advanced_equipment/rotorcraft");
	OtherworldMirrorActionUI.button({ translate: "item.megaverse:rotorcraft_extra_seat.name" },"textures/megaverse_items/default_equipment/advanced_equipment/rotorcraft_extra_seat");
	OtherworldMirrorActionUI.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		let selectedSummonId; let selectedSummonName; let requiredXP; let firstEvent; let secondEvent;
		if (response.selection==0) {
			selectedSummonId = "megaverse:motorbike";
			selectedSummonName = "item.megaverse:motorbike.name";
			requiredXP = 30;
			firstEvent = false;
			secondEvent = false;
		}
		if (response.selection==1) {
			selectedSummonId = "megaverse:motorbike";
			selectedSummonName = "item.megaverse:motorbike_extra_seat.name";
			requiredXP = 35;
			firstEvent = "minecraft:motorbike_extra_seat";
			secondEvent = false;
		}
		if (response.selection>=2) {
			selectedSummonId = undefined;
			selectedSummonName = undefined;
			requiredXP = 0;
			firstEvent = false;
			secondEvent = false;
		}
		if (selectedSummonId!=undefined) {
			source.playSound('vault.open_shutter',{location:source.location});
			OtherworldMirrorChosenSummoningUI(source,selectedSummonId,selectedSummonName,requiredXP,firstEvent,secondEvent);
		}
		else {
			source.playSound('vault.close_shutter',{location:source.location});
			OtherworldMirrorMainSummoningUI(source);
		}
	});
};

function OtherworldMirrorChosenSummoningUI(source,selectedSummonId,selectedSummonName,requiredXP,firstEvent,secondEvent) {
	const form = new MessageFormData();
	form.title({ translate: "ui.megaverse:otherworld_mirror.summon.title"});
	form.body(
		{rawtext:[
			{translate:"ui.megaverse:otherworld_mirror.summon.confirmation.required.text1"},
			{text:":"},{text:"\n§6§l"},
			{text:"\""},
			{translate:`${selectedSummonName}`},
			{text:"\""},{text:"§r\n"},
			{translate:"ui.megaverse:otherworld_mirror.summon.confirmation.required.text2"},
			{text:"\n"},{text:"\n§6§l"},
			{translate:`${requiredXP}`},
			{text:" "},{text:"§r"},
			{translate:"ui.megaverse:otherworld_mirror.summon.confirmation.required.text3"},
			{text:"\n"},
			{translate:"ui.megaverse:otherworld_mirror.summon.confirmation.required.text4"}
		]}
	);
	form.button1({ translate: "ui.megaverse:otherworld_mirror.summon.confirmation.false" });
	form.button2({ translate: "ui.megaverse:otherworld_mirror.summon.confirmation.true" });
	form.show(source).then((response) => {
		if (response.canceled) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			return source.playSound('vault.deactivate',{location:source.location});
		}
		if (response.selection==1) {
			if (source.getGameMode()!=GameMode.creative) {
				if (source.level<requiredXP) {
					source.sendMessage({rawtext:[{translate:`${source.name}`},{text:", "},{translate:`${requiredXP}`},{text:" "},{translate:"ui.megaverse:otherworld_mirror.summon.requirements"}]});
					return source.playSound('vault.deactivate',{location:source.location});
				}
				source.addLevels(-requiredXP);
			}
			source.playSound('block.end_portal.spawn',{location:source.location});
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.summon.confirmation"});
			let itemInQuestion = source.dimension.spawnEntity(selectedSummonId, {
				x: source.location.x+(source.getViewDirection().x),
				y: source.location.y+0.7+(source.getViewDirection().y),
				z: source.location.z+(source.getViewDirection().z)
			})
			itemInQuestion.addEffect("invisibility",32,{showParticles:false});
			itemInQuestion.addEffect("slowness",32,{amplifier:255,showParticles: false});
			itemInQuestion.addEffect("resistance",32,{amplifier:255,showParticles: false});
			itemInQuestion.dimension.spawnParticle("megaverse:otherworld_mirror_summoning", itemInQuestion.location);
			source.camera.fade({
				fadeColor: { red: 1, green: 1, blue: 1 },
				fadeTime: { fadeInTime: 1.60, holdTime: 0.15, fadeOutTime: 0.25 }
			});
			itemInQuestion.setRotation({x: 0, y: source.getRotation().y-180});
			itemInQuestion.triggerEvent(firstEvent);
			itemInQuestion.triggerEvent(secondEvent);
		}
		if (response.selection==0) {
			source.sendMessage({translate: "ui.megaverse:otherworld_mirror.cancellation"});
			source.playSound('vault.close_shutter',{location:source.location});
			OtherworldMirrorMainSummoningUI(source);
		}
	});
};