import { world, BlockPermutation, EntityDamageCause, EquipmentSlot, GameMode, ItemStack } from "@minecraft/server";
import * as blockUtilities from "./utilities/block_utilities.js";
import "./modules/block_modules.js";
import "./modules/entity_modules.js";
import "./modules/item_modules.js";
import "./modules/ui_modules.js";
import "./modules/world_modules.js";

world.beforeEvents.worldInitialize.subscribe(({blockComponentRegistry,itemComponentRegistry}) => {
	blockComponentRegistry.registerCustomComponent("megaverse:alchemy_altar_blocks", AlchemyAltarBlocks);
	blockComponentRegistry.registerCustomComponent("megaverse:alchemy_altar_interaction", AlchemyAltarInteraction);
	itemComponentRegistry.registerCustomComponent("megaverse:durability_damage_item_default", DurabilityDamageItemDefault);
	itemComponentRegistry.registerCustomComponent("megaverse:durability_damage_item_tool", DurabilityDamageItemTool);
	itemComponentRegistry.registerCustomComponent("megaverse:durability_damage_item_weapon", DurabilityDamageItemWeapon);
});

const randomInt = (min, max) => Math.floor(Math.random() * (max - min + 1)) + min;
const airBlocks = ["minecraft:air", "minecraft:fern", "minecraft:large_fern", "minecraft:short_grass", "minecraft:tall_grass", "minecraft:nether_sprouts", "minecraft:crimson_roots", "minecraft:warped_roots", "minecraft:snow_layer"];
const alchemic_books_list = [
	"megaverse:arcane_alchemic_book",
	"megaverse:aurora_alchemic_book",
	"megaverse:dark_alchemic_book",
	"megaverse:electric_alchemic_book",
	"megaverse:essentia_alchemic_book",
	"megaverse:fire_alchemic_book",
	"megaverse:geo_alchemic_book",
	"megaverse:ice_alchemic_book",
	"megaverse:light_alchemic_book",
	"megaverse:necromancy_alchemic_book",
	"megaverse:restoration_alchemic_book",
	"megaverse:soul_alchemic_book",
	"megaverse:stellar_alchemic_book",
	"megaverse:water_alchemic_book"
];
const alchemic_books_to_convert_list = {
	"megaverse:arcane_alchemic_book": "megaverse:arcane_alchemic_book",
	"megaverse:aurora_alchemic_book": "megaverse:aurora_alchemic_book",
	"megaverse:dark_alchemic_book": "megaverse:dark_alchemic_book",
	"megaverse:electric_alchemic_book": "megaverse:electric_alchemic_book",
	"megaverse:essentia_alchemic_book": "megaverse:essentia_alchemic_book",
	"megaverse:fire_alchemic_book": "megaverse:fire_alchemic_book",
	"megaverse:geo_alchemic_book": "megaverse:geo_alchemic_book",
	"megaverse:ice_alchemic_book": "megaverse:ice_alchemic_book",
	"megaverse:light_alchemic_book": "megaverse:light_alchemic_book",
	"megaverse:necromancy_alchemic_book": "megaverse:necromancy_alchemic_book",
	"megaverse:restoration_alchemic_book": "megaverse:restoration_alchemic_book",
	"megaverse:soul_alchemic_book": "megaverse:soul_alchemic_book",
	"megaverse:stellar_alchemic_book": "megaverse:stellar_alchemic_book",
	"megaverse:water_alchemic_book": "megaverse:water_alchemic_book",
};
const soundPitch = [1.00, 0.80, 1.10, 0.90, 1.20, 0.95, 1.15, 0.85, 1.05 ];
let defaultPitch = 0.00;
const AlchemyAltarBlocks = {
	onTick({block,dimension}) {
		const blockCanBeUsed = block.permutation.getState("megaverse:can_be_used");
		const blockHasBeenUsed = block.permutation.getState("megaverse:has_altar_used");
		const blockAbove = block.dimension.getBlock(block.above(1));
		const blockBelow = block.dimension.getBlock(block.below(1));
		const blockNorth = block.dimension.getBlock(block.offset({x:0,y:0,z:-1}));
		const blockSouth = block.dimension.getBlock(block.offset({x:0,y:0,z:1}));
		const blockWest = block.dimension.getBlock(block.offset({x:-1,y:0,z:0}));
		const blockEast = block.dimension.getBlock(block.offset({x:1,y:0,z:0}));
		if (blockCanBeUsed == false) {
			if (blockAbove.isAir && blockNorth.isAir && blockSouth.isAir && blockWest.isAir && blockEast.isAir) {
				if (blockBelow.hasTag("silver_block")) {
					block.setPermutation(block.permutation.withState("megaverse:can_be_used", true));
					dimension.playSound("beacon.activate", block.center());
				}
			}
		}
		if (blockCanBeUsed == true && blockHasBeenUsed == false) {
			if (!blockAbove.isAir||!blockNorth.isAir||!blockSouth.isAir||!blockWest.isAir||!blockEast.isAir) {
				block.setPermutation(block.permutation.withState("megaverse:can_be_used", false));
				dimension.playSound("beacon.deactivate", block.center());
			}
			if (!blockBelow.hasTag("silver_block")) {
				block.setPermutation(block.permutation.withState("megaverse:can_be_used", false));
				dimension.playSound("beacon.deactivate", block.center());
			}
		}
		if (blockHasBeenUsed == true) {
			block.setPermutation(block.permutation.withState("megaverse:has_altar_used", false));
			dimension.playSound("conduit.deactivate", block.center());
		}
	}
};
const AlchemyAltarInteraction = {
	onPlayerInteract({block,dimension,face,player}) {
		const blockCanBeUsed = block.permutation.getState("megaverse:can_be_used");
		const blockHasBeenUsed = block.permutation.getState("megaverse:has_altar_used");
		const blocksFace = face==="Up";
		const equippable = player.getComponent("minecraft:equippable");
		const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
		const itemStack = mainhand.getItem();
		const alchemicBooksConvert = alchemic_books_to_convert_list[itemStack.typeId];
		if (!mainhand.hasItem()) return;
		if (blockCanBeUsed == true && blockHasBeenUsed == false) {
			if (blocksFace) {
				if (alchemic_books_list.includes(itemStack.typeId)) {
					block.setPermutation(block.permutation.withState("megaverse:has_altar_used", true));
					dimension.spawnEntity(alchemicBooksConvert, block.above(1));
					dimension.playSound("conduit.activate", block.above(1));
					if (mainhand.amount > 1) mainhand.amount--;
					else mainhand.setItem(undefined);
				}
			}
		}
	}
};
const XpRewardBlockOreDefault = {
	onPlayerDestroy({block,dimension,player}) {
		const equippable = player?.getComponent("minecraft:equippable");
		if (!equippable) return;
		const itemStack = equippable.getEquipment(EquipmentSlot.Mainhand);
		if (player.getGameMode() !== GameMode.creative) {
			const enchantable = itemStack.getComponent("minecraft:enchantable");
			const silkTouch = enchantable?.getEnchantment("silk_touch");
			if (silkTouch) return;
			const xpAmount = randomInt(1, 3);
			for (let i = 0; i < xpAmount; i++) {
				dimension.spawnEntity("minecraft:xp_orb", block.location);
			}
		}
	}
};
const DurabilityDamageItemDefault = {
	onMineBlock({source}) {
		const equippable = source.getComponent("minecraft:equippable");
		if (!equippable) return;
		const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.getGameMode() === GameMode.creative) return;
		const itemStack = mainhand.getItem();
		const durability = itemStack.getComponent("minecraft:durability");
		if (!durability) return;
		const enchantable = itemStack.getComponent("minecraft:enchantable");
		const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level;
		const damageChance = durability.getDamageChance(unbreakingLevel) / 100;
		if (Math.random() > damageChance) return;
		const shouldBreak = durability.damage === durability.maxDurability;
		if (shouldBreak) {
			mainhand.setItem(undefined);
			source.playSound("random.break");
		}
		else {
			durability.damage++;
			mainhand.setItem(itemStack);
		}
	}
};
const DurabilityDamageItemTool = {
	onBeforeDurabilityDamage({attackingEntity,durabilityDamage}) {
		const equippable = attackingEntity.getComponent("minecraft:equippable");
		const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
		if (attackingEntity.getGameMode() === GameMode.creative) return;
		const itemStack = mainhand.getItem();
		const durability = itemStack.getComponent("minecraft:durability");
		if (!durability) return;
		const enchantable = itemStack.getComponent("minecraft:enchantable");
		const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level;
		const damageChance = durability.getDamageChance(unbreakingLevel) / 100;
		if (Math.random() > damageChance) return;
		const shouldBreak = durability.damage === durability.maxDurability;
		if (shouldBreak||durability.damage==1) {
			mainhand.setItem(undefined);
			source.playSound("random.break");
		}
		else {
			durability.damage++;
			durability.damage++;
			mainhand.setItem(itemStack);
		}
	},
	onMineBlock({source,minedBlockPermutation}) {
		const brokenBlockInQuestion = minedBlockPermutation;
		const equippable = source.getComponent("minecraft:equippable");
		if (!equippable) return;
		const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.getGameMode() === GameMode.creative) return;
		const itemStack = mainhand.getItem();
		const durability = itemStack.getComponent("minecraft:durability");
		if (!durability) return;
		const enchantable = itemStack.getComponent("minecraft:enchantable");
		const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level;
		const damageChance = durability.getDamageChance(unbreakingLevel) / 100;
		if (Math.random() > damageChance) return;
		const shouldBreak = durability.damage === durability.maxDurability;
		if (brokenBlockInQuestion.hasTag("minecraft:is_axe_item_destructible")||brokenBlockInQuestion.hasTag("minecraft:is_hoe_item_destructible")||brokenBlockInQuestion.hasTag("minecraft:is_pickaxe_item_destructible")||brokenBlockInQuestion.hasTag("minecraft:is_shovel_item_destructible")) {
			if (shouldBreak) {
				mainhand.setItem(undefined);
				source.playSound("random.break");
			}
			else {
				durability.damage++;
				mainhand.setItem(itemStack);
			}
		}
	}
};
const DurabilityDamageItemWeapon = {
	onMineBlock({source,minedBlockPermutation}) {
		const brokenBlockInQuestion = minedBlockPermutation;
		const equippable = source.getComponent("minecraft:equippable");
		if (!equippable) return;
		const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
		if (!mainhand.hasItem()) return;
		if (source.getGameMode() === GameMode.creative) return;
		const itemStack = mainhand.getItem();
		const durability = itemStack.getComponent("minecraft:durability");
		if (!durability) return;
		const enchantable = itemStack.getComponent("minecraft:enchantable");
		const unbreakingLevel = enchantable?.getEnchantment("unbreaking")?.level;
		const damageChance = durability.getDamageChance(unbreakingLevel) / 100;
		if (Math.random() > damageChance) return;
		const shouldBreak = durability.damage === durability.maxDurability;
		if (brokenBlockInQuestion.hasTag("minecraft:is_mace_item_destructible")||brokenBlockInQuestion.hasTag("minecraft:is_sword_item_destructible")) {
			if (shouldBreak||durability.damage==1) {
				mainhand.setItem(undefined);
				source.playSound("random.break");
			}
			else {
				durability.damage++;
				durability.damage++;
				mainhand.setItem(itemStack);
			}
		}
	}
};
world.sendMessage({translate:"pack.addon_script.work"});