import { world, system, BlockPermutation, EntityDamageCause, EquipmentSlot, GameMode, ItemStack } from "@minecraft/server";
import * as blockArrays from "../arrays/block_arrays.js"
import * as itemArrays from "../arrays/item_arrays.js"
import * as itemUtilities from "./item_utilities.js"
import * as mathUtilities from "./math_utilities.js"

export function blockChairSatDefault(block,dimension,player,tileEntityId,sitHeight) {
	const cardinalDirection = block.permutation.getState("minecraft:cardinal_direction");
	const listOfEntities = block.dimension.getEntitiesAtBlockLocation(block.location);
	const obstructiveBlocks = block.dimension.getBlock(block.location).above(1);
	for (const tileEntities of listOfEntities) {
		if (tileEntities.typeId === tileEntityId) {
			player.sendMessage({translate:"tile.megaverse:chair.occupied"});
			return;
		}
	}
	if (obstructiveBlocks.isAir==false) {
		player.sendMessage({translate:"tile.megaverse:chair.obstruction"});
		return;
	}
	const tileEntity = block.dimension.spawnEntity(tileEntityId,{x:block.center().x,y:block.location.y+sitHeight,z:block.center().z});
	const entityRideable = tileEntity.getComponent("minecraft:rideable");
	if (cardinalDirection=="north") {
		tileEntity.setRotation({x:0,y:180});
	}
	else if (cardinalDirection=="west") {
		tileEntity.setRotation({x:0,y:90});
	}
	else if (cardinalDirection=="east") {
		tileEntity.setRotation({x:0,y:-90});
	}
	entityRideable.addRider(player);
};

export function blockCropInteract(block,dimension,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (!mainhand.hasItem()||mainhand.typeId!=="minecraft:bone_meal") return;
	const growth = block.permutation.getState("megaverse:growth");
	const maxGrowth = blockArrays.blockCropGrowths[block.typeId];
	function useItem(mainhand) {
		if (mainhand.amount>1) mainhand.amount--;
		else mainhand.setItem(undefined);
	};
	function useBoneMeal(block,dimension) {
		dimension.playSound("item.bone_meal.use", block.center());
		dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
	};
	if (player.getGameMode()==GameMode.creative) {
		useBoneMeal(block,dimension);
		if (blockArrays.blockCropGrowsInto.includes(block.typeId)) {
			if (block.typeId=="megaverse:hortensia_seeds") {
				const weight = [15,10,30,30,10,5];
				const index = blockCropRandomiseType(weight);
				block.setType(blockArrays.blockCropHortensiaBush[index]);
			}
		}
		else block.setPermutation(block.permutation.withState("megaverse:growth",maxGrowth));
	}
	else {
		let growth = block.permutation.getState("megaverse:growth");
		if (growth!=maxGrowth) {
			growth += mathUtilities.randomInteger(1,maxGrowth-growth);
			block.setPermutation(block.permutation.withState("megaverse:growth",growth));
			useItem(mainhand);
			useBoneMeal(block,dimension);
		}
		if (growth==maxGrowth) {
			if (blockArrays.blockCropGrowsInto.includes(block.typeId)) {
				useItem(mainhand);
				useBoneMeal(block,dimension);
				if (block.typeId=="megaverse:hortensia_seeds") {
					const weight = [15,10,30,30,10,5];
					const index = blockCropRandomiseType(weight);
					block.setType(blockArrays.blockCropHortensiaBush[index]);
				}
			}
		}
	}
};

export function blockCropRandomiseType(weight) {
	const total = weight.reduce((a,b)=>a+b,0);
	const rand = Math.random()*total;
	let sum = 0;
	for (let i = 0; i < weight.length; i++) {
		sum += weight[i];
		if (rand<sum) return i;
	}
};

export function blockCropRandomTick(block,dimension) {
	const growth = block.permutation.getState("megaverse:growth");
	const blockBelow = block.dimension.getBlock(block.below(1));
	const moisturized = blockBelow.permutation.getState("moisturized_amount");
	const maxGrowth = blockArrays.blockCropGrowths[block.typeId];
	const percentageChance = (moisturized+1)/64;
	if (Math.random()>percentageChance) return;
	if (growth!=maxGrowth) block.setPermutation(block.permutation.withState("megaverse:growth",growth+1));
	if (growth==maxGrowth) {
		if (block.typeId=="megaverse:hortensia_seeds") {
			const weight = [15,10,30,30,10,5];
			const index = blockCropRandomiseType(weight);
			block.setType(blockArrays.blockCropHortensiaBush[index]);
		}
	}
};

export function blockEggBreak(block,destroyedBlockPermutation,dimension,player) {
	if (player.getGameMode()==GameMode.creative) return;
	const rule = blockArrays.blockEggBreakTypes[destroyedBlockPermutation.type.id];
	if (!rule) return;
	const {amountCount,crackedState,sound} = rule;
	const amount = destroyedBlockPermutation.getState(amountCount.state);
	const cracks = destroyedBlockPermutation.getState(crackedState.state);
	const isString = amountCount.type=="string";
	if (sound) dimension.playSound(sound,block.center());
	if (player.isSneaking) {
		if (isString) {
			const index = amountCount.values.indexOf(amount);
			if (index>0) {
				const newAmount = amountCount.values[index-1];
				block.setPermutation(BlockPermutation.resolve(destroyedBlockPermutation.type.id,{[amountCount.state]:newAmount,[crackedState.state]:cracks}));
			}
		}
		else if (amount>amountCount.min) {
			block.setPermutation(BlockPermutation.resolve(destroyedBlockPermutation.type.id,{[amountCount.state]:amount-1,[crackedState.state]:cracks}));
		}
		return;
	}
	let dropCount = 0;
	if (isString) {
		dropCount = amountCount.values.indexOf(amount);
	}
	else {
		dropCount = amount;
	}
	for (let i = 0; i < dropCount; i++) {
		dimension.spawnItem(new ItemStack(destroyedBlockPermutation.type.id,1),block.center());
	}
};

export function blockEggFallOn(block,dimension) {
	const rule = blockArrays.blockEggFallOnTypes[block.typeId];
	if (!rule) return;
	const breakChance = 1/rule.breakDenominator;
	if (Math.random()>breakChance) {
		if (rule.crackParticle) dimension.spawnParticle(rule.crackParticle,block.center());
		if (rule.crackSound) dimension.playSound(rule.crackSound,block.center());
		return;
	}
	const {count} = rule;
	const amount = block.permutation.getState(count.state);
	let changeAmount = false;
	if (count.type=="string") {
		const index = count.values.indexOf(amount);
		if (index>0) {
			block.setPermutation(block.permutation.withState(count.state,count.values[index-1]));
			changeAmount = true;
		}
	}
	else {
		if (amount>count.min) {
			block.setPermutation(block.permutation.withState(count.state,amount-1));
			changeAmount = true;
		}
	}
	if (rule.breakParticle) dimension.spawnParticle(rule.breakParticle,block.center());
	if (rule.breakSound) dimension.playSound(rule.breakSound,block.center());
	if (!changeAmount) {
		block.setType(rule.breakInto);
	}
};

export function blockEggInteract(block,dimension,player) {
	const rule = blockArrays.blockEggInteractTypes[block.typeId];
	if (!rule) return;
	const permutation = block.permutation;
	const itemStack = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (itemStack.typeId==block.typeId) {
		if (player.getGameMode()!=GameMode.creative) {
			if (itemStack.amount>1) itemStack.amount--;
			else itemStack.setItem(undefined);
		}
		const {type} = rule.interaction;
		const blockState = permutation.getState(rule.interaction.state);
		if (rule.particle) dimension.spawnParticle(rule.particle,block.center());
		if (rule.sound) dimension.playSound(rule.sound,block.center());
		if (type=="string") {
			const index = rule.interaction.values.indexOf(blockState);
			if (index<rule.interaction.values.length-1) {
				block.setPermutation(permutation.withState(rule.interaction.state,rule.interaction.values[index+1]));
				return;
			}
		}
		else if (type=="integer") {
			if (blockState<rule.interaction.max) {
				block.setPermutation(permutation.withState(rule.interaction.state,blockState+1));
				return;
			}
		}
	}
	else return;
};

export function blockEggTick(block,dimension) {
	const rule = blockArrays.blockEggHatchTypes[block.typeId];
	const face = block.permutation.getState("minecraft:block_face");
	const array = blockArrays;
	if (!rule||!rule.check(block,face,array)) return;
	const permutation = block.permutation;
	const {type} = rule.hatching;
	if (type=="string") {
		const hatchState = permutation.getState(rule.hatching.state);
		const index = rule.hatching.values.indexOf(hatchState);
		if (index<rule.hatching.values.length-1) {
			if (rule.crackParticle) dimension.spawnParticle(rule.crackParticle,block.center());
			if (rule.crackSound) dimension.playSound(rule.crackSound, block.center());
			block.setPermutation(permutation.withState(rule.hatching.state,rule.hatching.values[index+1]));
			return;
		}
	}
	else if (type=="integer") {
		const hatchState = permutation.getState(rule.hatching.state);
		if (hatchState<rule.hatching.max) {
			if (rule.crackParticle) dimension.spawnParticle(rule.crackParticle,block.center());
			if (rule.crackSound) dimension.playSound(rule.crackSound, block.center());
			block.setPermutation(permutation.withState(rule.hatching.state,hatchState+1));
			return;
		}
	}
	if (rule.hatchParticle) dimension.spawnParticle(rule.hatchParticle,rule.entitySpawnBlockFace?rule.entitySpawnUsingBlockFace(block,permutation,rule.entitySpawnBlockFace):rule.entitySpawnAt(block));
	if (rule.hatchSound) dimension.playSound(rule.hatchSound,rule.entitySpawnBlockFace?rule.entitySpawnUsingBlockFace(block,permutation,rule.entitySpawnBlockFace):rule.entitySpawnAt(block));
	if (rule.blockTurnsInto) {
		if (rule.blockTurnsWithBlockFace) block.setPermutation(BlockPermutation.resolve(rule.blockTurnsInto,{[rule.blockTurnsWithBlockFace]:rule.blockIntoBlockFaceState(permutation,rule.blockTurnsWithBlockFace)}));
		else block.setType(rule.blockTurnsInto);
	}
	let hatchCount = typeof rule.count==="function"?rule.count(mathUtilities):rule.count;
	if (rule.addCount.boolean==true) {
		const amount = permutation.getState(rule.addCount.state);
		if (rule.addCount.type=="string") {
			const indexAmount = rule.addCount.values.indexOf(amount);
			if (indexAmount!=-1) hatchCount += indexAmount;
		}
		else hatchCount += amount;
	}
	for (let i = 0; i < hatchCount; i++) {
		let entitySpawned = dimension.spawnEntity(`${rule.entity}<${rule.event}>`,rule.entitySpawnBlockFace?rule.entitySpawnUsingBlockFace(block,permutation,rule.entitySpawnBlockFace):rule.entitySpawnAt(block));
		if (rule.addEvents.value==true) {
			entitySpawned.triggerEvent(rule.addEvents.event1);
		}
	}
};

export function blockFlowerInteract(block,dimension,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (!mainhand.hasItem()||mainhand.typeId!=="minecraft:bone_meal") return;
	dimension.playSound("item.bone_meal.use", block.center());
	dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
	if (player.getGameMode()!=GameMode.creative) {
		if (mainhand.amount>1) mainhand.amount--;
		else mainhand.setItem(undefined);
	}
	const blockName = block.typeId.split("megaverse:");
	if (blockArrays.blockFlowerGrowsInto[block.typeId]) {
		dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
		return block.setType(blockArrays.blockFlowerGrowsInto[block.typeId]);
	}
	else {
		dimension.spawnParticle("minecraft:crop_growth_area_emitter", block.center());
		return dimension.placeFeature(`megaverse:bone_meal_grow_${blockName[1]}_feature`,block.center());
	}
};

export function blockSaplingInteract(block,dimension,player) {
	const rule = blockArrays.blockSaplingTypes[block.typeId];
	if (!rule) return;
	const itemStack = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (!rule.growSaplingWith.includes(itemStack.typeId)) return;
	if (rule.growParticle) dimension.spawnParticle(rule.growParticle,rule.growSaplingAt(block));
	if (rule.growSound) dimension.playSound(rule.growSound,rule.growSaplingAt(block));
	if (player.getGameMode()!=GameMode.creative) itemStack.amount>1?itemStack.amount--:itemStack.setItem(undefined);
	if (player.getGameMode()==GameMode.creative) {
		system.run(()=>{
			dimension.placeFeature(rule.growSaplingInto,rule.growSaplingAt(block));
		});
	}
	else {
		const fractionChance = 1/rule.chanceDenominator;
		if (Math.random()>fractionChance) return;
		system.run(()=>{
			dimension.placeFeature(rule.growSaplingInto,rule.growSaplingAt(block));
		});
	}
};

export function blockSaplingRandomTick(block,dimension) {
	const rule = blockArrays.blockSaplingTypes[block.typeId];
	if (!rule) return;
	const fractionChance = 1/rule.growthDenominator;
	if (Math.random()>fractionChance) return;
	system.run(()=>{
		dimension.placeFeature(rule.growSaplingInto,rule.growSaplingAt(block));
	});
};

export function blockSmallShutterInteract(block,dimension,player) {
	const blockType = block.typeId;
	const openBit = block.permutation.getState("megaverse:open_bit");
	if (openBit==false) {
		block.setPermutation(block.permutation.withState("megaverse:open_bit",true));
		if (blockType.includes("bamboo")||blockType.includes("cherry")||blockType.includes("crimson")||blockType.includes("warped")) {
			if (blockType.includes("bamboo")) dimension.playSound("open.bamboo_wood_trapdoor", block.center());
			if (blockType.includes("cherry")) dimension.playSound("open.cherry_wood_trapdoor", block.center());
			if (blockType.includes("crimson")||blockType.includes("warped")) dimension.playSound("open.nether_wood_trapdoor", block.center());
		}
		else dimension.playSound("open.wooden_trapdoor", block.center());
	}
	else {
		block.setPermutation(block.permutation.withState("megaverse:open_bit",false));
		if (blockType.includes("bamboo")||blockType.includes("cherry")||blockType.includes("crimson")||blockType.includes("warped")) {
			if (blockType.includes("bamboo")) dimension.playSound("close.bamboo_wood_trapdoor", block.center());
			if (blockType.includes("cherry")) dimension.playSound("close.cherry_wood_trapdoor", block.center());
			if (blockType.includes("crimson")||blockType.includes("warped")) dimension.playSound("close.nether_wood_trapdoor", block.center());
		}
		else dimension.playSound("close.wooden_trapdoor", block.center());
	}
};

export function blockTrapCoverFallOn(block,dimension) {
	const isTrapCover = (typeId)=>blockArrays.blockTrapCoverTypes.includes(typeId);
	const offsets = [
		{x: 0, z: 0},
		{x: 1, z: 0}, {x: -1, z: 0}, {x: 0, z: 1}, {x: 0, z: -1},
		{x: 1, z: 1}, {x: -1, z: -1}, {x: 1, z: -1}, {x: -1, z: 1}
	];
	if (block.typeId.includes("leaf")) dimension.playSound("block.leaf_litter.break", block.center());
	else if (block.typeId.includes("moss")) dimension.playSound("dig.moss", block.center());
	else dimension.playSound("dig.grass", block.center());
	for (const offset of offsets) {
		const x = block.location.x+offset.x;
		const y = block.location.y;
		const z = block.location.z+offset.z;
		const adjacent = dimension.getBlock({x,y,z});
		if (adjacent&&isTrapCover(adjacent.typeId)) {
			dimension.runCommand(`setblock ${x} ${y} ${z} air destroy`);
		}
	}
};

export function blockTrapCoverStepOn(block, dimension, entity) {
	//const rule = blockArrays.blockTrapCoverTypes[block.typeId];
	const isTrapCover = (typeId)=>blockArrays.blockTrapCoverTypes.includes(typeId);
	const offsets = [
		{x: 0, z: 0},
		{x: 1, z: 0}, {x: -1, z: 0}, {x: 0, z: 1}, {x: 0, z: -1},
		{x: 1, z: 1}, {x: -1, z: -1}, {x: 1, z: -1}, {x: -1, z: 1}
	];
	dimension.spawnParticle(`${block.typeId}_breaking`, {x:block.center().x,y:block.location.y+1,z:block.center().z});
	if (entity.isSneaking) {
		if (block.typeId.includes("leaf")) dimension.playSound("block.leaf_litter.step", block.center());
		else if (block.typeId.includes("moss")) dimension.playSound("step.moss", block.center());
		else dimension.playSound("step.grass", block.center());
		return;
	}
	if (block.typeId.includes("leaf")) dimension.playSound("block.leaf_litter.break", block.center());
	else if (block.typeId.includes("moss")) dimension.playSound("dig.moss", block.center());
	else dimension.playSound("dig.grass", block.center());
	if (entity.typeId=="minecraft:item") return;
	for (const offset of offsets) {
		const x = block.location.x+offset.x;
		const y = block.location.y;
		const z = block.location.z+offset.z;
		const adjacent = dimension.getBlock({x,y,z});
		if (adjacent&&isTrapCover(adjacent.typeId)) {
			dimension.runCommand(`setblock ${x} ${y} ${z} air destroy`);
		}
	}
};

export function cryothermaliaRandomTick(block,dimension) {
	const blockBelow = block.dimension.getBlock(block.below(1));
	const active = block.permutation.getState("megaverse:active");
	const convertBlock = blockBelow.typeId.split("minecraft:");
	const toFrost = "megaverse:frosted_";
	if (active==false) {
		if (blockBelow.typeId.includes("frosted_")||blockBelow.typeId.includes("glaciered_")) {
			block.setPermutation(block.permutation.withState("megaverse:active",true));
			dimension.playSound("mob.player.hurt_freeze", block.center());
			dimension.spawnParticle("megaverse:cryothermalia_freeze", block.center());
		}
		else {
			blockBelow.setPermutation(BlockPermutation.resolve(`${toFrost}${convertBlock[1]}`,{"megaverse:age":0}));
		}
	}
	else {
		if (!blockBelow.typeId.includes("frosted_")&&!blockBelow.typeId.includes("glaciered_")) {
			block.setPermutation(block.permutation.withState("megaverse:active",false));
			dimension.playSound("random.glass", block.center());
			dimension.spawnParticle("megaverse:frost", block.center());
		}
	}
};

export function cryothermaliaTick(block,dimension) {
	const mobs = block.dimension.getEntities({families:["mob"],location:block.center(),maxDistance:1});
	const players = block.dimension.getEntities({type:"player",location:block.center(),maxDistance:1});
	mobs.forEach((mob) => {
		const mobType = mob.typeId;
		if (mobType.includes("frost")||mobType.includes("ice_")) return;
		dimension.spawnParticle("megaverse:cryothermalia_freeze",block.center());
		mob.applyDamage(1,{cause:EntityDamageCause.freezing});
	});
	players.forEach((player) => {
		dimension.spawnParticle("megaverse:cryothermalia_freeze",block.center());
		player.applyDamage(1,{cause:EntityDamageCause.freezing});
	});
};

export function deerSkullBreak(block,destroyedBlockPermutation,dimension,player) {
	if (player.getGameMode()==GameMode.creative) return;
	const amount = destroyedBlockPermutation.getState("megaverse:amount");
	const antler = amount-1;
	if (amount==1) return dimension.spawnItem(new ItemStack("megaverse:deer_antler",1),block.center());
	else for (let i = 0; i < antler; i++) dimension.spawnItem(new ItemStack("megaverse:deer_antler",1),block.center());
};

export function deerSkullInteract(block,dimension,face,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	const amount = block.permutation.getState("megaverse:amount");
	const rotation = block.permutation.getState("megaverse:rotation");
	const direction = block.permutation.getState("minecraft:cardinal_direction");
	const blockFace = block.permutation.getState("minecraft:block_face");
	function interactSound(block,dimension) {
		dimension.playSound("dig.bone_block", block.center());
	};
	function decrementStack(mainhand,player) {
		if (player.getGameMode()!=GameMode.creative) {
			if (mainhand.amount>1) mainhand.amount--;
			else mainhand.setItem(undefined);
		}
	};
	function leftAntler(block,dimension,mainhand,player) {
		interactSound(block,dimension);
		decrementStack(mainhand,player);
		block.setPermutation(block.permutation.withState("megaverse:amount",1));
	};
	function rightAntler(block,dimension,mainhand,player) {
		interactSound(block,dimension);
		decrementStack(mainhand,player);
		block.setPermutation(block.permutation.withState("megaverse:amount",2));
	};
	function allAntlers(block,dimension,mainhand,player) {
		interactSound(block,dimension);
		decrementStack(mainhand,player);
		block.setPermutation(block.permutation.withState("megaverse:amount",3));
	};
	if (!mainhand.hasItem()||mainhand.typeId!="megaverse:deer_antler") {
		interactSound(block,dimension);
		if (rotation!=4) return block.setPermutation(block.permutation.withState("megaverse:rotation",rotation+1));
		else return block.setPermutation(block.permutation.withState("megaverse:rotation",0));
	}
	else if (mainhand.typeId=="megaverse:deer_antler") {
		if (amount==0) {
			if (blockFace=="up"||blockFace=="down") {
				if (direction=="north") {
					if (rotation<=2) {
						if (face=="West") return leftAntler(block,dimension,mainhand,player);
						if (face=="East") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==3) {
						if (face=="West") return leftAntler(block,dimension,mainhand,player);
						if (face=="North") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==4) {
						if (face=="North") return leftAntler(block,dimension,mainhand,player);
						if (face=="East") return rightAntler(block,dimension,mainhand,player);
					}
				}
				if (direction=="south") {
					if (rotation<=2) {
						if (face=="East") return leftAntler(block,dimension,mainhand,player);
						if (face=="West") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==3) {
						if (face=="East") return leftAntler(block,dimension,mainhand,player);
						if (face=="South") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==4) {
						if (face=="South") return leftAntler(block,dimension,mainhand,player);
						if (face=="West") return rightAntler(block,dimension,mainhand,player);
					}
				}
				if (direction=="west") {
					if (rotation<=2) {
						if (face=="South") return leftAntler(block,dimension,mainhand,player);
						if (face=="North") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==3) {
						if (face=="South") return leftAntler(block,dimension,mainhand,player);
						if (face=="West") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==4) {
						if (face=="West") return leftAntler(block,dimension,mainhand,player);
						if (face=="North") return rightAntler(block,dimension,mainhand,player);
					}
				}
				if (direction=="east") {
					if (rotation<=2) {
						if (face=="North") return leftAntler(block,dimension,mainhand,player);
						if (face=="South") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==3) {
						if (face=="North") return leftAntler(block,dimension,mainhand,player);
						if (face=="East") return rightAntler(block,dimension,mainhand,player);
					}
					if (rotation==4) {
						if (face=="East") return leftAntler(block,dimension,mainhand,player);
						if (face=="South") return rightAntler(block,dimension,mainhand,player);
					}
				}
			}
			if (blockFace=="north") {
				if (face=="West") return leftAntler(block,dimension,mainhand,player);
				if (face=="East") return rightAntler(block,dimension,mainhand,player);
			}
			if (blockFace=="south") {
				if (face=="East") return leftAntler(block,dimension,mainhand,player);
				if (face=="West") return rightAntler(block,dimension,mainhand,player);
			}
			if (blockFace=="west") {
				if (face=="South") return leftAntler(block,dimension,mainhand,player);
				if (face=="North") return rightAntler(block,dimension,mainhand,player);
			}
			if (blockFace=="east") {
				if (face=="North") return leftAntler(block,dimension,mainhand,player);
				if (face=="South") return rightAntler(block,dimension,mainhand,player);
			}
		}
		if (amount==1) {
			if (blockFace=="up"||blockFace=="down") {
				if (direction=="north") {
					if (rotation!=3&&face=="East") return allAntlers(block,dimension,mainhand,player);
					if (rotation==3&&face=="North") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="south") {
					if (rotation!=3&&face=="West") return allAntlers(block,dimension,mainhand,player);
					if (rotation==3&&face=="South") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="west") {
					if (rotation!=3&&face=="North") return allAntlers(block,dimension,mainhand,player);
					if (rotation==3&&face=="West") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="east") {
					if (rotation!=3&&face=="South") return allAntlers(block,dimension,mainhand,player);
					if (rotation==3&&face=="East") return allAntlers(block,dimension,mainhand,player);
				}
			}
			if (blockFace=="north"&&face=="East") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="south"&&face=="West") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="west"&&face=="North") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="east"&&face=="South") return allAntlers(block,dimension,mainhand,player);
		}
		if (amount==2) {
			if (blockFace=="up"||blockFace=="down") {
				if (direction=="north") {
					if (rotation!=4&&face=="West") return allAntlers(block,dimension,mainhand,player);
					if (rotation==4&&face=="North") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="south") {
					if (rotation!=4&&face=="East") return allAntlers(block,dimension,mainhand,player);
					if (rotation==4&&face=="South") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="west") {
					if (rotation!=4&&face=="South") return allAntlers(block,dimension,mainhand,player);
					if (rotation==4&&face=="West") return allAntlers(block,dimension,mainhand,player);
				}
				if (direction=="east") {
					if (rotation!=4&&face=="North") return allAntlers(block,dimension,mainhand,player);
					if (rotation==4&&face=="East") return allAntlers(block,dimension,mainhand,player);
				}
			}
			if (blockFace=="north"&&face=="West") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="south"&&face=="East") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="west"&&face=="South") return allAntlers(block,dimension,mainhand,player);
			if (blockFace=="east"&&face=="North") return allAntlers(block,dimension,mainhand,player);
		}
	}
};
export function firewoodBreak(block,destroyedBlockPermutation,dimension,player) {
	if (player.getGameMode()==GameMode.creative) return;
	const amount = destroyedBlockPermutation.getState("megaverse:amount");
	for (let i = 0; i < amount; i++) {
		dimension.spawnItem(new ItemStack(destroyedBlockPermutation.type.id,1),block.center());
	}
};

export function firewoodInteract(block,dimension,player) {
	const item = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	const amount = block.permutation.getState("megaverse:amount");
	const direction = block.permutation.getState("minecraft:cardinal_direction");
	const fuelItemTags = itemArrays.firewoodKindlingFuelItemTags.some(tag=>item.hasTag(tag));
	const fuelItemTypes = itemArrays.firewoodKindlingFuelItemTypes.includes(item.typeId);
	if (item.typeId==block.typeId&&amount!=15) {
		itemUtilities.itemDecrementStack(item,player);
		dimension.playSound("dig.wood", block.center());
		block.setPermutation(block.permutation.withState("megaverse:amount",amount+1));
	}
	if (amount==3&&(fuelItemTags||fuelItemTypes)) {
		itemUtilities.itemDecrementStack(item,player);
		dimension.playSound("use.wood", block.center());
		block.setPermutation(BlockPermutation.resolve("megaverse:kindling_pile",{"minecraft:cardinal_direction":direction,"megaverse:active":false}));
	}
};

export function frostRandomTick(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	const percentageChance = 1/3;
	if (Math.random()>percentageChance) return;
	if (age!=3) block.setPermutation(block.permutation.withState("megaverse:age",age+1));
	else {
		block.setType("minecraft:air");
		dimension.playSound("break.suspicious_sand", block.center());
		dimension.spawnParticle(block.typeId, block.center());
	}
};

export function frostTick(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	const blockBelow = block.dimension.getBlock(block.below(1));
	const convertBlock = block.typeId.split(":");
	if (blockBelow.isAir||blockBelow.isLiquid) {
		block.setType("minecraft:air");
		let fallingBlock = dimension.spawnEntity(`${convertBlock[0]}:falling_${convertBlock[1]}`,{x:block.center().x,y:block.location.y,z:block.center().z});
		fallingBlock.setProperty("megaverse:falling_block_age",age);
	}
	else return;
};

export function frostedBlockRandomTick(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	const convertBlock = block.typeId.split("frosted_");
	if (age!=3) block.setPermutation(block.permutation.withState("megaverse:age",age+1));
	else block.setType(`minecraft:${convertBlock[1]}`);
};
export function frostedBlockTick(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	const blockBelow = block.dimension.getBlock(block.below(1));
	const convertBlock = block.typeId.split(":");
	if (blockBelow.isAir||blockBelow.isLiquid) {
		block.setType("minecraft:air");
		let fallingBlock = dimension.spawnEntity(`${convertBlock[0]}:falling_${convertBlock[1]}`,{x:block.center().x,y:block.location.y,z:block.center().z});
		fallingBlock.setProperty("megaverse:falling_block_age",age);
	}
	else return;
};
export function iceSpikesFallOn(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	if (age!=3) {
		dimension.spawnParticle("megaverse:frost", block.center());
		dimension.playSound("random.glass", block.center());
		block.setPermutation(block.permutation.withState("megaverse:age",age+1));
	}
	else {
		dimension.spawnParticle("megaverse:frost", block.center());
		dimension.playSound("random.glass", block.center());
		block.setType("minecraft:air");
	}
};

export function iceSpikesRandomTick(block,dimension) {
	const age = block.permutation.getState("megaverse:age");
	if (age!=3) {
		dimension.playSound("drip.water.pointed_dripstone", block.center());
		block.setPermutation(block.permutation.withState("megaverse:age",age+1));
	}
	else {
		dimension.spawnParticle("megaverse:frost", block.center());
		dimension.playSound("drip.water.pointed_dripstone", block.center());
		block.setType("minecraft:air");
	}
};

export function iceSpikesStepOn(block,dimension,entity) {
	const age = block.permutation.getState("megaverse:age");
	const damageAmount = 4-age;
	entity.applyDamage(damageAmount,{cause:EntityDamageCause.contact});
	if (age!=3) {
		dimension.spawnParticle("megaverse:frost", block.center());
		dimension.playSound("random.glass", block.center());
		block.setPermutation(block.permutation.withState("megaverse:age",age+1));
	}
	else {
		dimension.spawnParticle("megaverse:frost", block.center());
		dimension.playSound("random.glass", block.center());
		block.setType("minecraft:air");
	}
};

export function kindlingPileBreak(block,destroyedBlockPermutation,dimension,player) {
	if (player.getGameMode()==GameMode.creative) return;
	const active = destroyedBlockPermutation.getState("megaverse:active");
	if (active==true) {
		dimension.spawnItem(new ItemStack("minecraft:stick",1),block.center());
	}
};

export function kindlingPileInteract(block,dimension,player) {
	const item = player.getComponent("minecraft:equippable").getEquipment(EquipmentSlot.Mainhand);
	const enchantable = item.getComponent("minecraft:enchantable");
	const fireAspect = enchantable?.getEnchantment("fire_aspect");
	const active = block.permutation.getState("megaverse:active");
	const amount = block.permutation.getState("megaverse:amount");
	const direction = block.permutation.getState("minecraft:cardinal_direction");
	if (item.typeId=="minecraft:stick") {
		if (active==false) {
			itemUtilities.itemDecrementStack(item,player);
			dimension.playSound("dig.wood", block.center());
			block.setPermutation(block.permutation.withState("megaverse:active",true));
		}
		else {
			const percentageChance = 1/3;
			if (Math.random()>percentageChance) {
				dimension.spawnParticle("megaverse:fire_starting_smoke",block.center());
				dimension.playSound("fire.ignite",block.center());
			}
			else {
				const percentageChance = 1/2;
				if (Math.random()>percentageChance) {
					itemUtilities.itemDecrementStack(item,player);
					dimension.spawnParticle("megaverse:fire_starting_smoke",block.center());
					dimension.playSound("random.break",block.center());
					block.setPermutation(block.permutation.withState("megaverse:active",false));
				}
				else {
					dimension.spawnParticle("megaverse:fire_starting_spark",block.center());
					dimension.playSound("mob.ghast.fireball",block.center());
					block.setPermutation(BlockPermutation.resolve("minecraft:campfire",{"minecraft:cardinal_direction":direction,"extinguished":false}));
				}
			}
		}
	}
	if (item.typeId=="minecraft:flint") {
		const percentageChance = 1/3;
		if (Math.random()>percentageChance) {
			dimension.spawnParticle("megaverse:fire_starting_smoke",block.center());
			dimension.playSound("fire.ignite",block.center());
		}
		else {
			dimension.spawnParticle("megaverse:fire_starting_spark",block.center());
			dimension.playSound("mob.ghast.fireball",block.center());
			block.setPermutation(BlockPermutation.resolve("minecraft:campfire",{"minecraft:cardinal_direction":direction,"extinguished":false}));
		}
	}
	if (fireAspect||item.typeId=="minecraft:fire_charge"||item.typeId=="minecraft:flint_and_steel") {
		block.setPermutation(BlockPermutation.resolve("minecraft:campfire",{"minecraft:cardinal_direction":direction,"extinguished":false}));
		dimension.spawnParticle("megaverse:fire_starting_spark", block.center());
		dimension.playSound("fire.ignite", block.center());
	}
};

export function rockFernInteract(block,dimension,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (!mainhand.hasItem()||mainhand.typeId!=="minecraft:bone_meal") return;
	dimension.playSound("item.bone_meal.use", block.center());
	dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
	if (player.getGameMode()!=GameMode.creative) {
		if (mainhand.amount>1) mainhand.amount--;
		else mainhand.setItem(undefined);
	}
	if (block.typeId=="megaverse:short_rock_fern") return block.setPermutation(BlockPermutation.resolve("megaverse:tall_rock_fern"));
	else {
		dimension.spawnParticle("minecraft:crop_growth_area_emitter", block.center());
		dimension.placeFeature("megaverse:bone_meal_grow_short_rock_fern_feature",block.center());
	}
};

export function sconceInteract(block,dimension,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	const extinguished = block.permutation.getState("megaverse:extinguished");
	const blockFace = block.permutation.getState("minecraft:block_face");
	const offset = blockArrays.sconceParticleOffsets[blockFace];
	const center = block.center();
	const vector = {x:center.x+offset.x,y:center.y+offset.y,z:center.z+offset.z};
	const itemStack = mainhand.getItem();
	function matchesFireItem(itemStack) {
		return (itemArrays.blockFireItemsTypeIncludes.some(substr => itemStack.typeId.includes(substr))||itemArrays.blockFireItemsTypeId.includes(itemStack.typeId)||itemArrays.blockFireItemsTags.some(tag => itemStack.hasTag(tag)));
	};
	if (extinguished==true) {
		if (matchesFireItem(itemStack)) {
			player.playSound("mob.ghast.fireball",{location:center,pitch:1.25});
			dimension.spawnParticle("megaverse:sconce_emitting_smoke",vector);
			return block.setPermutation(block.permutation.withState("megaverse:extinguished",false));
		}
	}
	else {
		if (mainhand.hasItem()) return;
		player.playSound("random.fizz",{location:center});
		dimension.spawnParticle("megaverse:sconce_extinguished_smoke",vector);
		return block.setPermutation(block.permutation.withState("megaverse:extinguished",true));
	}
};
export function sconceRandomTick(block,dimension) {
	const extinguished = block.permutation.getState("megaverse:extinguished");
	const blockFace = block.permutation.getState("minecraft:block_face");
	const offset = blockArrays.sconceParticleOffsets[blockFace];
	const center = block.center();
	const vector = {x:center.x+offset.x,y:center.y+offset.y,z:center.z+offset.z};
	if (extinguished==true) return;
	else return dimension.spawnParticle("megaverse:sconce_emitting_smoke",vector);
};

export function silkwormCocoonTick(block,dimension) {
	const array = blockArrays;
	const rule = array.silkwormCocoonTypes[block.typeId];
	const face = block.permutation.getState("minecraft:block_face");
	if (!rule||!rule.check(block)) return;
	const permutation = block.permutation;
	if (rule.condition.type=="bool") {
		const conditionState = permutation.getState(rule.condition.state);
		if (conditionState==false) {
			if (rule.addition.type="string") {
				const additionState = permutation.getState(rule.addition.state);
				const index = rule.addition.values.indexOf(additionState);
				if (index>0) {
					if (rule.turnParticle) dimension.spawnParticle(rule.turnParticle,block.center());
					if (rule.turnSound) dimension.playSound(rule.turnSound,block.center());
					return block.setPermutation(permutation.withState(rule.addition.state,rule.addition.values[index-1]));
				}
				else if (index==0&&rule.criteria(block)) {
					if (rule.turnParticle) dimension.spawnParticle(rule.turnParticle,block.center());
					if (rule.turnSound) dimension.playSound(rule.turnSound,block.center());
					return block.setPermutation(permutation.withState(rule.condition.state,true));
				}
				else return;
			}
		}
		else if (conditionState==true) {
			if (rule.turnParticle) dimension.spawnParticle(rule.turnParticle,block.center());
			if (rule.turnSound) dimension.playSound(rule.turnSound,block.center());
			if (rule.criteria(block)) {
				if (rule.blockTurnsWithBlockFace) return block.setPermutation(BlockPermutation.resolve(rule.blockTurnsInto,{[rule.blockTurnsWithBlockFace]:rule.blockIntoBlockFaceState(permutation,rule.blockTurnsWithBlockFace)}));
				else block.setType(rule.blockTurnsInto);
			}
			else return block.setPermutation(permutation.withState(rule.condition.state,false));
		}
	}
};