import { world, system, EntityDamageCause, Player } from "@minecraft/server";
import * as dragonArrays from "../arrays/dragon_arrays.js"
import * as mathUtilities from "../utilities/math_utilities.js"

export const dragonTypes = { families: ["dragon"] };

export function dragonsMainComponents(dragon) {
	const dragonOnList = dragonArrays.dragonTypesList.includes(dragon.typeId);
	if (!dragonOnList) return;
	const rule = dragonArrays.dragonTypes[dragon.typeId];
	if (!rule) return;
	if (dragon.getComponent("minecraft:is_baby")) return;
	const isCooldown = dragon.getProperty("megaverse:mob_has_spitting_cooldown");
	const isSpitting = dragon.getProperty("megaverse:mob_is_spitting");
	const navigationState = dragon.getProperty("megaverse:mob_navigation_state");
	const isOnGround = dragon.getProperty("megaverse:is_on_ground");
	if (isCooldown) dragon.addEffect("absorption",100,{amplifier:1,showParticles:true});
	const rideable = dragon.getComponent("rideable");
	if (!rideable) return;
	const controllingSeat = rideable.controllingSeat;
	//if (controllingSeat == undefined) return;
	const riders = rideable.getRiders();
	if (!riders) return;
	const controllingRider = riders[controllingSeat];
	if (controllingRider instanceof Player) {
		handleDragonFire(dragon,controllingRider,isSpitting,isCooldown);
		//handleDragonInputControl(rule,dragon,controllingRider,navigationState,isOnGround);
		handleDragonPitchControl(rule,dragon,controllingRider,navigationState,isOnGround);
	}
};

function handleDragonFire(dragon,player,isSpitting,isCooldown) {
	if (player.isJumping&&!isSpitting&&!isCooldown) {
		dragon.triggerEvent("minecraft:dracon_spit_start");
		player.addEffect("absorption",100,{amplifier:1,showParticles:true});
	}
};

function handleDragonInputControl(rule,dragon,player,navigationState,isOnGround) {
	const movement = player.inputInfo.getMovementVector();
	if (!movement) return;
	const normalizedX = Math.abs(movement.x) >= rule.threshold ? (movement.x > 0 ? 1 : -1) : 0;
	const normalizedY = Math.abs(movement.y) >= rule.threshold ? (movement.y > 0 ? 1 : -1) : 0;
	const inputKey = `${normalizedX},${normalizedY}`;
	const inputType = rule.inputDirections[inputKey] ?? "noAcceleration";
	const acceleration = rule[inputType]??0;
	// Get forward direction
	const forward = dragon.getViewDirection();
	// Calculate right vector (perpendicular to forward vector on XZ plane)
	const right = { x: forward.z, y: 0, z: -forward.x };
	// Calculate final direction vector from input
	let moveDir = { x: 0, y: 0, z: 0 };
	if (normalizedY !== 0) {
		moveDir.x += forward.x * normalizedY;
		moveDir.z += forward.z * normalizedY;
	}
	if (normalizedX !== 0) {
		moveDir.x += right.x * normalizedX;
		moveDir.z += right.z * normalizedX;
	}
	// Normalize moveDir to prevent faster diagonals
	const magnitude = Math.hypot(moveDir.x, moveDir.z);
	if (magnitude > 0) {
		moveDir.x /= magnitude;
		moveDir.z /= magnitude;
	}
	// Apply scale depending on ground status
	const scale = dragon.isInWater?rule.movementOnWater:isOnGround?rule.movementOnGround:rule.movementOnAir;
	// Final impulse
	dragon.applyImpulse({
		x: moveDir.x*acceleration*scale,
		y: 0,
		z: moveDir.z*acceleration*scale
	});
};

function handleDragonPitchControl(rule,dragon,player,navigationState,isOnGround) {
	player.playAnimation("animation.megaverse.dracon.rotations_rider",{stopExpression: "!q.is_riding||v.is_first_person"});
	const pitch = player.getRotation().x;
	let altitude = 0;
	if (pitch<-22.5) {
		altitude = 0.03;
		if (navigationState!="air_controlled") dragon.triggerEvent("minecraft:dragon_air_controlled");
	}
	else if (pitch>35&&!isOnGround) altitude = -0.03;
	else if (pitch>=35&&isOnGround&&navigationState!="grounded") dragon.triggerEvent("minecraft:dragon_grounded");
	else altitude = -dragon.getVelocity().y;
	if (altitude!=0) return dragon.applyImpulse({x:0,y:altitude,z:0});
};
/* Note: Max Y Velocity for Dracon base is 0.6
player.sendMessage(`VC: ${Math.round(dragon.getVelocity().y*10)/10}`);
*/

function lerp(a,b,t) {
	return a+(b-a)*t;
};