import { world, system, EntityDamageCause, EntityInventoryComponent, EquipmentSlot, GameMode, ItemStack, Player } from "@minecraft/server";

export function alchemyWandCompleteUse(itemStack,source) {
	const mainhand = source.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	const durability = itemStack.getComponent("minecraft:durability");
	const enchantable = itemStack.getComponent("minecraft:enchantable");
	if (source.getGameMode()==GameMode.creative) return;
	const unbreaking = enchantable?.getEnchantment("unbreaking")?.level;
	const damageChance = 1/(unbreaking+1);
	if (Math.random()>damageChance) return;
	if (durability.damage>=durability.maxDurability) {
		mainhand.setItem(new ItemStack("megaverse:empty_alchemy_wand"));
		source.playSound("magic.aurora_magic.default_impact");
	}
	else {
		durability.damage++;
		source.playSound("break.amethyst_cluster");
		mainhand.setItem(itemStack);
	}
};
export function draconAcidBottleConsume(itemStack,source) {
	if (source.getGameMode()==GameMode.creative) return;
	source.applyDamage(1,{cause:EntityDamageCause.fireTick});
	source.addEffect("nausea",200);
	source.dimension.spawnParticle("minecraft:critical_hit_emitter",{x:source.location.x,y:source.location.y+1,z:source.location.z});
	source.playSound("mob.player.hurt_on_fire");
	const health = source.getComponent("health");
	let interval;
	function looper() {
		if (health.currentValue==0||health.currentValue==health.effectiveMax||source.getGameMode()==GameMode.creative) return system.clearRun(interval);
		system.runTimeout(() => {
			return system.clearRun(interval);
		},30);
		source.applyDamage(1,{cause:EntityDamageCause.fireTick});
		source.dimension.spawnParticle("minecraft:critical_hit_emitter",{x:source.location.x,y:source.location.y+1,z:source.location.z});
		source.playSound("mob.player.hurt_on_fire");
	};
	interval = system.runInterval(looper,10);
};
export function hadrosauridHorn(itemStack,source) {
	source.startItemCooldown("megaverse:hadrosaurid_horn",60);
	source.playSound("item.instrument.hadrosaurid_horn",{location:source.location});
	let hornCall = source.dimension.spawnEntity(`${itemStack.typeId}_call`, {
		x: source.location.x+(source.getViewDirection().x==0),
		y: source.location.y+1.4+(source.getViewDirection().y==0),
		z: source.location.z+(source.getViewDirection().z==0)
	});
	let callComponent = hornCall.getComponent("minecraft:projectile");
	callComponent.owner = source;
	callComponent.shoot({
		x: source.getViewDirection().x*5,
		y: source.getViewDirection().y*5,
		z: source.getViewDirection().z*5
	});
	return hornCall;
};
export function iceWyvernVenomousFrostBottleConsume(itemStack,source) {
	if (source.getGameMode()==GameMode.creative) return;
	source.applyDamage(1,{cause:EntityDamageCause.freezing});
	source.dimension.spawnParticle("megaverse:frost",{x:source.location.x,y:source.location.y+1,z:source.location.z});
	source.playSound("mob.player.hurt_freeze");
	const health = source.getComponent("health");
	let interval;
	function looper() {
		if (health.currentValue==0||health.currentValue==health.effectiveMax||source.getGameMode()==GameMode.creative) return system.clearRun(interval);
		system.runTimeout(() => {
			if (health.currentValue!=0&&health.currentValue!=health.effectiveMax&&source.getGameMode()!=GameMode.creative) source.addEffect("fatal_poison",80);
			return system.clearRun(interval);
		},30);
		source.applyDamage(1,{cause:EntityDamageCause.freezing});
		source.dimension.spawnParticle("megaverse:frost",{x:source.location.x,y:source.location.y+1,z:source.location.z});
		source.playSound("mob.player.hurt_freeze");
	};
	interval = system.runInterval(looper,10);
};
export function itemDecrementStack(itemStack,player) {
	if (player.getGameMode()==GameMode.creative) return;
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	if (itemStack.amount>1) mainhand.amount--;
	else mainhand.setItem(undefined);
};
export function itemToolDamage(itemStack,player) {
	const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
	const durability = itemStack.getComponent("minecraft:durability");
	const enchantable = itemStack.getComponent("minecraft:enchantable");
	if (!durability||!enchantable) return;
	if (player.getGameMode()==GameMode.creative) return;
	const unbreaking = enchantable?.getEnchantment("unbreaking")?.level;
	const damageChance = 1/(unbreaking+1);
	if (Math.random()>damageChance) return;
	if (durability.damage>=durability.maxDurability) {
		mainhand.setItem(undefined);
		player.playSound("random.break");
	}
	else {
		durability.damage++;
		mainhand.setItem(itemStack);
	}
};
export function magicSphereProjectile(itemStack,source,soundType,entityType,entityEvent,magicType,castTimer,velocity) {
	source.playSound(soundType,{location:source.location});
	let projectileEntity = source.dimension.spawnEntity(`${entityType}${entityEvent}`, {
		x: source.location.x+(source.getViewDirection().x==0),
		y: source.location.y+1.4+(source.getViewDirection().y==0),
		z: source.location.z+(source.getViewDirection().z==0)
	});
	projectileEntity.triggerEvent(magicType);
	projectileEntity.triggerEvent(castTimer);
	let projectileType = projectileEntity.getComponent("minecraft:projectile");
	projectileType.owner = source;
	projectileType.shoot({
		x: source.getViewDirection().x*velocity,
		y: source.getViewDirection().y*velocity,
		z: source.getViewDirection().z*velocity
	});
	return projectileEntity;
};
export function rawMeatConsume(itemStack,source) {
	const percentageChance = 3/10;
	if (Math.random()>percentageChance) return;
	source.addEffect("hunger",600);
};